@@ -1,20 +1,22 @@
-from logging import getLogger
+from copy import deepcopy
 from logging import NullHandler
+from logging import getLogger
 from os import environ
-from pkg_resources import parse_version
 from time import sleep
 
 try:  # python 3
     from collections.abc import Iterable
+    from collections.abc import Mapping
     from urllib.parse import urljoin
     from urllib.parse import urlparse
 except ImportError:  # python 2
     from collections import Iterable
+    from collections import Mapping
     from urlparse import urljoin
     from urlparse import urlparse
 
+from pkg_resources import parse_version
 from requests import exceptions as requests_exceptions
-from requests import head as requests_head
 from requests import Session
 from requests.adapters import HTTPAdapter
 from six import string_types as six_string_types
@@ -53,9 +55,9 @@ class HelpersMixIn(object):
         :param delimiter: delimiter for concatenation
         :return: if input is a list, concatenated string...else whatever the input was
         """
-        if not isinstance(input_list, six_string_types) and isinstance(
-            input_list, Iterable
-        ):
+        is_string = isinstance(input_list, six_string_types)
+        is_iterable = isinstance(input_list, Iterable)
+        if is_iterable and not is_string:
             return delimiter.join(map(str, input_list))
         return input_list
 
@@ -95,7 +97,9 @@ class HelpersMixIn(object):
 
 
 class Request(HelpersMixIn):
-    """Facilitates HTTP requests to qBittorrent."""
+    """
+    Facilitates HTTP requests to qBittorrent's Web API.
+    """
 
     def __init__(self, host="", port=None, username=None, password=None, **kwargs):
         self.host = host
@@ -119,14 +123,16 @@ class Request(HelpersMixIn):
         This is necessary on startup or when the auth cookie needs to be replaced...perhaps
         because it expired, qBittorrent was restarted, significant settings changes, etc.
         """
+        logger.debug("Re-initializing context...")
         # base path for all API endpoints
         self._API_BASE_PATH = "api/v2"
 
-        # reset URL so the full URL is derived again (primarily allows for switching scheme for WebUI: HTTP <-> HTTPS)
+        # reset URL so the full URL is derived again
+        # (primarily allows for switching scheme for WebUI: HTTP <-> HTTPS)
         self._API_BASE_URL = None
 
         # reset Requests session so it is rebuilt with new auth cookie and all
-        self._requests_session = None
+        self._trigger_session_initialization()
 
         # reinitialize interaction layers
         self._application = None
@@ -143,6 +149,7 @@ class Request(HelpersMixIn):
     def _initialize_lesser(
         self,
         EXTRA_HEADERS=None,
+        REQUESTS_ARGS=None,
         VERIFY_WEBUI_CERTIFICATE=True,
         FORCE_SCHEME_FROM_HOST=False,
         RAISE_UNIMPLEMENTEDERROR_FOR_UNIMPLEMENTED_API_ENDPOINTS=False,
@@ -156,7 +163,8 @@ class Request(HelpersMixIn):
         """Initialize lessor used configuration"""
 
         # Configuration parameters
-        self._EXTRA_HEADERS = EXTRA_HEADERS if isinstance(EXTRA_HEADERS, dict) else {}
+        self._EXTRA_HEADERS = EXTRA_HEADERS or {}
+        self._REQUESTS_ARGS = REQUESTS_ARGS or {}
         self._VERIFY_WEBUI_CERTIFICATE = bool(VERIFY_WEBUI_CERTIFICATE)
         self._VERBOSE_RESPONSE_LOGGING = bool(VERBOSE_RESPONSE_LOGGING)
         self._PRINT_STACK_FOR_EACH_REQUEST = bool(PRINT_STACK_FOR_EACH_REQUEST)
@@ -213,11 +221,11 @@ class Request(HelpersMixIn):
             http_method="post", api_namespace=_name, api_method=_method, **kwargs
         )
 
-    def _request_manager(self, _retries=2, _retry_backoff_factor=0.3, **kwargs):
+    def _request_manager(self, _retries=1, _retry_backoff_factor=0.3, **kwargs):
         """
         Wrapper to manage request retries and severe exceptions.
 
-        This should retry at least twice to account for the Web API switching from HTTP to HTTPS.
+        This should retry at least once to account for the Web API switching from HTTP to HTTPS.
         During the second attempt, the URL is rebuilt using HTTP or HTTPS as appropriate.
         """
 
@@ -244,27 +252,29 @@ class Request(HelpersMixIn):
             return err_msg
 
         def retry_backoff(retry_count):
-            """Back off on attempting each subsequent request retry."""
+            """
+            Back off on attempting each subsequent request retry.
+            The first retry is always immediate. if the backoff factor is 0.3,
+            then will sleep for 0s then .3s, then .6s, etc. between retries.
+            """
             if retry_count > 0:
-                # The first retry is always immediate. if the backoff factor is 0.3,
-                # then will sleep for 0s then .3s, then .6s, etc. between retries.
                 backoff_time = _retry_backoff_factor * (2 ** ((retry_count + 1) - 1))
                 sleep(backoff_time if backoff_time <= 10 else 10)
-            logger.debug("Retry attempt %d", (retry_count + 1))
+            logger.debug("Retry attempt %d", retry_count + 1)
 
-        max_retries = _retries if _retries > 1 else 2
+        max_retries = _retries if _retries >= 1 else 1
         for retry in range(0, (max_retries + 1)):
             try:
                 return self._request(**kwargs)
-            except HTTPError as e:
+            except HTTPError as exc:
                 # retry the request for HTTP 500 statuses;
                 # raise immediately for other HTTP errors (e.g. 4XX statuses)
-                if retry >= max_retries or not isinstance(e, HTTP5XXError):
+                if retry >= max_retries or not isinstance(exc, HTTP5XXError):
                     raise
-            except Exception as e:
+            except Exception as exc:
                 if retry >= max_retries:
-                    error_message = build_error_msg(exc=e)
-                    response = getattr(e, "response", None)
+                    error_message = build_error_msg(exc=exc)
+                    response = getattr(exc, "response", None)
                     raise APIConnectionError(error_message, response=response)
 
             retry_backoff(retry_count=retry)
@@ -280,18 +290,80 @@ class Request(HelpersMixIn):
         :param kwargs: see _normalize_requests_params for additional support
         :return: Requests response
         """
-        url = self._build_url(api_namespace=api_namespace, api_method=api_method)
-        kwargs = self._trim_known_kwargs(kwargs=kwargs)
-        params = self._normalize_requests_params(http_method=http_method, **kwargs)
+        api_args, requests_args = self._normalize_args(http_method, **kwargs)
+        url = self._build_url(api_namespace, api_method, requests_args=requests_args)
 
-        response = self._session.request(method=http_method, url=url, **params)
+        http_args = api_args.copy()
+        http_args.update(requests_args)
+        response = self._session.request(http_method, url, **http_args)
 
-        self.verbose_logging(http_method=http_method, response=response, url=url)
-        self.handle_error_responses(params=params, response=response)
+        self._verbose_logging(http_method, url, http_args, response)
+        self._handle_error_responses(api_args, response)
         return response
 
+    def _normalize_args(
+        self,
+        http_method,
+        headers=None,
+        data=None,
+        params=None,
+        files=None,
+        requests_params=None,
+        requests_args=None,
+        **kwargs
+    ):
+        """
+        Extract several keyword arguments to send in the request.
+
+        :param headers: additional headers to send with the request
+        :param data: key/value pairs to send as body
+        :param params: key/value pairs to send as query parameters
+        :param files: key/value pairs to include as multipart POST requests
+        :param requests_params: original name for requests_args
+        :param requests_args: keyword arguments for call to Requests
+        :return: dictionary of parameters for Requests call
+        """
+        kwargs = self._trim_known_kwargs(**kwargs)
+
+        # these are completely user defined and intended to allow users
+        # of this client to control the behavior of Requests
+        override_requests_args = self._get_requests_args(
+            headers=headers,
+            requests_params=requests_params,
+            requests_args=requests_args,
+        )
+
+        # these are expected to be populated by this client as necessary for qBittorrent
+        data = data or {}
+        params = params or {}
+        files = files or {}
+
+        # send Content-Length as 0 for empty POSTs...Requests will not send Content-Length
+        # if data is empty but qBittorrent will complain otherwise
+        is_data_content = any(filter(lambda x: x is None, data.values()))
+        if http_method.lower() == "post" and not is_data_content:
+            override_requests_args.setdefault("headers", {})
+            override_requests_args["headers"].update({"Content-Length": "0"})
+
+        # any other keyword arguments are sent to qBittorrent as part of the request.
+        # These are user-defined since this Client will put everything in data/params/files
+        # that needs to be sent to qBittorrent.
+        if kwargs:
+            if http_method == "get":
+                params.update(kwargs)
+            if http_method == "post":
+                data.update(kwargs)
+
+        # arguments specified during Client construction
+        requests_args = deepcopy(self._REQUESTS_ARGS)
+        # incorporate arguments specified for this specific API call
+        requests_args.update(override_requests_args)
+        # qbittorrent api call arguments
+        api_args = dict(data=data, params=params, files=files)
+        return api_args, requests_args
+
     @staticmethod
-    def _trim_known_kwargs(kwargs):
+    def _trim_known_kwargs(**kwargs):
         """
         Since any extra keyword arguments from the user are automatically
         included in the request to qBittorrent, this removes any "known"
@@ -306,7 +378,22 @@ class Request(HelpersMixIn):
         kwargs.pop("SIMPLE_RESPONSE", None)
         return kwargs
 
-    def _build_url(self, api_namespace, api_method):
+    @staticmethod
+    def _get_requests_args(**kwargs):
+        """
+        Return any user-supplied arguments for Requests.
+        Users are expected to specify requests-specific parameters in requests_args.
+        Headers have historically been treated specially and are supported as a
+        naked argument to the generalized request methods (i.e. _get, _post, _request).
+        Going forward, additional parameters for managing the behavior of Requests
+        should only be supported the requests_args parameter.
+        """
+        args = kwargs.get("requests_args") or kwargs.get("requests_params") or {}
+        if "headers" in kwargs:
+            args.setdefault("headers", {}).update(kwargs["headers"] or {})
+        return args
+
+    def _build_url(self, api_namespace, api_method, requests_args):
         """
         Create a fully qualified URL for the API endpoint.
 
@@ -314,137 +401,182 @@ class Request(HelpersMixIn):
         :param api_method: the specific method for the API endpoint (e.g. info)
         :return: urllib URL object
         """
-        self._API_BASE_URL = self._build_base_url(
+
+        def build_base_url(
+            client,
+            base_url=None,
+            host="",
+            port=None,
+            force_user_scheme=False,
+            requests_args=None,
+        ):
+            """
+            Determine the Base URL for the Web API endpoints.
+
+            A URL is only actually built here if it's the first time here or
+            the context was re-initialized. Otherwise, the most recently
+            built URL is used.
+
+            If the user doesn't provide a scheme for the URL, it will try HTTP
+            first and fall back to HTTPS if that doesn't work. While this is
+            probably backwards, qBittorrent or an intervening proxy can simply
+            redirect to HTTPS and that'll be respected.
+
+            Additionally, if users want to augment the path to the API endpoints,
+            any path provided here will be preserved in the returned Base URL
+            and prefixed to all subsequent API calls.
+
+            :param client: this larger qbittorrent client
+            :param base_url: if the URL was already built, this is the base URL
+            :param host: user provided hostname for WebUI
+            :param port: TCP port to attempt HTTP connection on
+            :param force_user_scheme: use the scheme in the host instead of detecting the proper scheme
+            :param requests_args: additional parameters from user for making HTTP HEAD request
+            :return: base URL for Web API endpoint
+            """
+            if base_url is not None:
+                return base_url
+
+            # Parse user host - urlparse requires some sort of schema for parsing to work at all
+            if not host.lower().startswith(("http:", "https:", "//")):
+                host = "//" + host
+            base_url = urlparse(url=host)
+            logger.debug("Parsed user URL: %r", base_url)
+
+            # default to HTTP if user didn't specify
+            user_scheme = base_url.scheme
+            default_scheme = user_scheme or "http"
+            alt_scheme = "https" if default_scheme == "http" else "http"
+
+            # add port number if host doesn't contain one
+            if port is not None and not base_url.port:
+                host = "".join((base_url.netloc, ":", str(port)))
+                base_url = base_url._replace(netloc=host)
+
+            # detect whether Web API is configured for HTTP or HTTPS
+            if not (user_scheme and force_user_scheme):
+                logger.debug("Detecting scheme for URL...")
+                prefer_https = False
+                for scheme in (default_scheme, alt_scheme):
+                    try:
+                        base_url = base_url._replace(scheme=scheme)
+                        r = client._session.request(
+                            "head", base_url.geturl(), **requests_args
+                        )
+                        scheme_to_use = urlparse(r.url).scheme
+                        break
+                    except requests_exceptions.SSLError:
+                        # an SSLError means that qBittorrent is likely listening on HTTPS
+                        # but the TLS connection is not trusted...so, if the attempt to
+                        # connect on HTTP also fails, this will tell us to switch back to HTTPS
+                        logger.debug(
+                            "Encountered SSLError: will prefer HTTPS if HTTP fails"
+                        )
+                        prefer_https = True
+                    except requests_exceptions.RequestException:
+                        logger.debug(
+                            "Failed connection attempt with %s", scheme.upper()
+                        )
+                else:
+                    scheme_to_use = "https" if prefer_https else "http"
+
+                # use detected scheme
+                logger.debug("Using %s scheme", scheme_to_use.upper())
+                base_url = base_url._replace(scheme=scheme_to_use)
+                if user_scheme and user_scheme != scheme_to_use:
+                    logger.warning(
+                        "Using '%s' instead of requested '%s' to communicate with qBittorrent",
+                        scheme_to_use,
+                        user_scheme,
+                    )
+
+            # ensure URL always ends with a forward-slash
+            base_url = base_url.geturl()
+            if not base_url.endswith("/"):
+                base_url = base_url + "/"
+            logger.debug("Base URL: %s", base_url)
+
+            # force a new session to be created now that the URL is known
+            client._trigger_session_initialization()
+
+            return base_url
+
+        def build_url_path(base_url, api_base_path, api_namespace, api_method):
+            """
+            Determine the full URL path for the API endpoint.
+
+            :param base_url: base URL for API (e.g. http://localhost:8080 or http://example.com/qbt/)
+            :param api_base_path: qBittorrent defined API path prefix (i.e. api/v2/)
+            :param api_namespace: the namespace for the API endpoint (e.g. torrents)
+            :param api_method: the specific method for the API endpoint (e.g. info)
+            :return: full urllib URL object for API endpoint
+                     (e.g. http://localhost:8080/api/v2/torrents/info or http://example.com/qbt/api/v2/torrents/info)
+            """
+
+            def sanitize(piece):
+                """Ensure each piece of api path is a string without leading or trailing slashes"""
+                return str(piece or "").strip("/")
+
+            if isinstance(api_namespace, APINames):
+                api_namespace = api_namespace.value
+            api_path = "/".join(
+                map(sanitize, (api_base_path, api_namespace, api_method))
+            )
+            # since base_url is guaranteed to end in a slash and api_path will never
+            # start with a slash, this join only ever append to the path in base_url
+            url = urljoin(base_url, api_path)
+            return url
+
+        self._API_BASE_URL = build_base_url(
+            client=self,
             base_url=self._API_BASE_URL,
             host=self.host,
             port=self.port,
             force_user_scheme=self._FORCE_SCHEME_FROM_HOST,
+            requests_args=requests_args,
         )
-        return self._build_url_path(
+        return build_url_path(
             base_url=self._API_BASE_URL,
             api_base_path=self._API_BASE_PATH,
             api_namespace=api_namespace,
             api_method=api_method,
         )
 
-    @staticmethod
-    def _build_base_url(base_url=None, host="", port=None, force_user_scheme=False):
-        """
-        Determine the Base URL for the Web API endpoints.
-
-        A URL is only actually built here if it's the first time here or
-        the context was re-initialized. Otherwise, the most recently
-        built URL is used.
-
-        If the user doesn't provide a scheme for the URL, it will try HTTP
-        first and fall back to HTTPS if that doesn't work. While this is
-        probably backwards, qBittorrent or an intervening proxy can simply
-        redirect to HTTPS and that'll be respected.
-
-        Additionally, if users want to augment the path to the API endpoints,
-        any path provided here will be preserved in the returned Base URL
-        and prefixed to all subsequent API calls.
-
-        :param base_url: if the URL was already built, this is the base URL
-        :param host: user provided hostname for WebUI
-        :return: base URL for Web API endpoint
-        """
-        if base_url is not None:
-            return base_url
-
-        # urlparse requires some sort of schema for parsing to work at all
-        if not host.lower().startswith(("http:", "https:", "//")):
-            host = "//" + host
-        base_url = urlparse(url=host)
-        logger.debug("Parsed user URL: %r", base_url)
-        # default to HTTP if user didn't specify
-        user_scheme = base_url.scheme
-        base_url = base_url._replace(scheme="http") if not user_scheme else base_url
-        alt_scheme = "https" if base_url.scheme == "http" else "http"
-        # add port number if host doesn't contain one
-        if port is not None and not isinstance(base_url.port, int):
-            base_url = base_url._replace(netloc="%s:%s" % (base_url.netloc, port))
-
-        # detect whether Web API is configured for HTTP or HTTPS
-        if not (user_scheme and force_user_scheme):
-            logger.debug("Detecting scheme for URL...")
-            try:
-                # skip verification here...if there's a problem, we'll catch it during the actual API call
-                r = requests_head(base_url.geturl(), allow_redirects=True, verify=False)
-                # if WebUI eventually supports sending a redirect from HTTP to HTTPS then
-                # Requests will automatically provide a URL using HTTPS.
-                # For instance, the URL returned below will use the HTTPS scheme.
-                #  >>> requests.head('http://grc.com', allow_redirects=True).url
-                scheme = urlparse(r.url).scheme
-            except requests_exceptions.RequestException:
-                # assume alternative scheme will work...we'll fail later if neither are working
-                scheme = alt_scheme
-
-            # use detected scheme
-            logger.debug("Using %s scheme", scheme.upper())
-            base_url = base_url._replace(scheme=scheme)
-            if user_scheme and user_scheme != scheme:
-                logger.warning(
-                    "Using '%s' instead of requested '%s' to communicate with qBittorrent",
-                    scheme,
-                    user_scheme,
-                )
-
-        # ensure URL always ends with a forward-slash
-        base_url = base_url.geturl()
-        if not base_url.endswith("/"):
-            base_url = base_url + "/"
-        logger.debug("Base URL: %s", base_url)
-
-        return base_url
-
-    @staticmethod
-    def _build_url_path(base_url, api_base_path, api_namespace, api_method):
-        """
-        Determine the full URL path for the API endpoint.
-
-        :param base_url: base URL for API (e.g. http://localhost:8080 or http://example.com/qbt/)
-        :param api_base_path: qBittorrent defined API path prefix (i.e. api/v2/)
-        :param api_namespace: the namespace for the API endpoint (e.g. torrents)
-        :param api_method: the specific method for the API endpoint (e.g. info)
-        :return: full urllib URL object for API endpoint
-                 (e.g. http://localhost:8080/api/v2/torrents/info or http://example.com/qbt/api/v2/torrents/info)
-        """
-
-        def sanitize(piece):
-            """Ensure each piece of api path is a string without leading or trailing slashes"""
-            return str(piece or "").strip("/")
-
-        if isinstance(api_namespace, APINames):
-            api_namespace = api_namespace.value
-        api_path = "/".join(map(sanitize, (api_base_path, api_namespace, api_method)))
-        # since base_url is guaranteed to end in a slash and api_path will never
-        # start with a slash, this join only ever append to the path in base_url
-        url = urljoin(base_url, api_path)
-        return url
-
     @property
     def _session(self):
         """
-        Create or return existing Requests session.
+        Create or return existing HTTP session.
 
         :return: Requests Session object
         """
-        if self._requests_session:
-            return self._requests_session
 
-        self._requests_session = Session()
+        class QbittorrentSession(Session):
+            """
+            Wrapper to augment Requests Session.
+            Requests doesn't allow Session to default certain configuration
+            globally. This gets around that by setting defaults for each request.
+            """
+
+            def request(self, *args, **kwargs):
+                kwargs.setdefault("timeout", 15.1)
+                kwargs.setdefault("allow_redirects", True)
+                return super(QbittorrentSession, self).request(*args, **kwargs)
+
+        if self._http_session:
+            return self._http_session
+
+        self._http_session = QbittorrentSession()
 
         # default headers to prevent qBittorrent throwing any alarms
-        self._requests_session.headers.update(
+        self._http_session.headers.update(
             {"Referer": self._API_BASE_URL, "Origin": self._API_BASE_URL}
         )
 
         # add any user-defined headers to be sent in all requests
-        self._requests_session.headers.update(self._EXTRA_HEADERS)
+        self._http_session.headers.update(self._EXTRA_HEADERS)
 
         # enable/disable TLS verification for all requests
-        self._requests_session.verify = self._VERIFY_WEBUI_CERTIFICATE
+        self._http_session.verify = self._VERIFY_WEBUI_CERTIFICATE
 
         # enable retries in Requests if HTTP call fails.
         # this is sorta doubling up on retries since request_manager() will
@@ -458,76 +590,45 @@ class Request(HelpersMixIn):
         # at any rate, the retries count in request_manager should always be
         # at least 2 to accommodate significant settings changes in qBittorrent
         # such as enabling HTTPs in Web UI settings.
-        retries = 1
-        retry = Retry(
-            total=retries,
-            read=retries,
-            connect=retries,
-            status_forcelist={500, 502, 504},
-            raise_on_status=False,
+        adapter = HTTPAdapter(
+            max_retries=Retry(
+                total=1,
+                read=1,
+                connect=1,
+                status_forcelist={500, 502, 504},
+                raise_on_status=False,
+            )
         )
-        adapter = HTTPAdapter(max_retries=retry)
-        self._requests_session.mount("http://", adapter)
-        self._requests_session.mount("https://", adapter)
+        self._http_session.mount("http://", adapter)
+        self._http_session.mount("https://", adapter)
 
-        return self._requests_session
+        return self._http_session
+
+    def __del__(self):
+        """
+        Close HTTP Session before destruction.
+        This isn't strictly necessary since this will automatically
+        happen when the Session is garbage collected...but it makes
+        Python's ResourceWarning logging for unclosed sockets cleaner.
+        """
+        self._trigger_session_initialization()
+
+    def _trigger_session_initialization(self):
+        """
+        Effectively resets the HTTP session by removing the reference to it.
+        During the next request, a new session will be created.
+        """
+        if hasattr(self, "_http_session") and isinstance(self._http_session, Session):
+            self._http_session.close()
+        self._http_session = None
 
     @staticmethod
-    def _normalize_requests_params(
-        http_method,
-        headers=None,
-        data=None,
-        params=None,
-        files=None,
-        requests_params=None,
-        **kwargs
-    ):
-        """
-        Extract several keyword arguments to send in the request.
-
-        :param headers: additional headers to send with the request
-        :param data: key/value pairs to send as body
-        :param params: key/value pairs to send as query parameters
-        :param files: key/value pairs to include as multipart POST requests
-        :param requests_params: keyword arguments for call to Requests
-        :return: dictionary of parameters for Requests call
-        """
-        # these are completely user defined and intended to allow users
-        # of this client to control the behavior of Requests
-        requests_params = requests_params or {}
-
-        # these are expected to be populated by this client as necessary for qBittorrent
-        data = data or {}
-        params = params or {}
-        files = files or {}
-
-        # these are user-defined headers to include with the request
-        headers = headers or {}
-        # send Content-Length as 0 for empty POSTs...Requests will not send Content-Length
-        # if data is empty but qBittorrent may complain otherwise
-        if http_method == "post" and not any(filter(None, data.values())):
-            headers["Content-Length"] = "0"
-
-        # any other keyword arguments are sent to qBittorrent as part of the request.
-        # These are user-defined since this Client will put everything in data/params/files
-        # that needs to be sent to qBittorrent.
-        if kwargs:
-            if http_method == "get":
-                params.update(kwargs)
-            if http_method == "post":
-                data.update(kwargs)
-
-        d = dict(data=data, params=params, files=files, headers=headers)
-        d.update(requests_params)
-        return d
-
-    @staticmethod
-    def handle_error_responses(params, response):
+    def _handle_error_responses(args, response):
         """Raise proper exception if qBittorrent returns Error HTTP Status."""
         if response.status_code < 400:
             # short circuit for non-error statuses
             return
-        elif response.status_code == 400:
+        if response.status_code == 400:
             # Returned for malformed requests such as missing or invalid parameters.
             #
             # If an error_message isn't returned, qBittorrent didn't receive all required parameters.
@@ -537,63 +638,60 @@ class Request(HelpersMixIn):
                 raise MissingRequiredParameters400Error()
             raise InvalidRequest400Error(response.text)
 
-        elif response.status_code == 401:
+        if response.status_code == 401:
             # Primarily reserved for XSS and host header issues. Is also
             raise Unauthorized401Error(response.text)
 
-        elif response.status_code == 403:
+        if response.status_code == 403:
             # Not logged in or calling an API method that isn't public
             # APIErrorType::AccessDenied
             raise Forbidden403Error(response.text)
 
-        elif response.status_code == 404:
+        if response.status_code == 404:
             # API method doesn't exist or more likely, torrent not found
             # APIErrorType::NotFound
             error_message = response.text
             if error_message in ("", "Not Found"):
                 error_torrent_hash = ""
-                if params["data"]:
-                    error_torrent_hash = params["data"].get("hash", error_torrent_hash)
-                    error_torrent_hash = params["data"].get(
-                        "hashes", error_torrent_hash
-                    )
-                if params and error_torrent_hash == "":
-                    error_torrent_hash = params["params"].get(
-                        "hash", error_torrent_hash
-                    )
-                    error_torrent_hash = params["params"].get(
+                if args["data"]:
+                    error_torrent_hash = args["data"].get("hash", error_torrent_hash)
+                    error_torrent_hash = args["data"].get("hashes", error_torrent_hash)
+                if args and error_torrent_hash == "":
+                    error_torrent_hash = args["params"].get("hash", error_torrent_hash)
+                    error_torrent_hash = args["params"].get(
                         "hashes", error_torrent_hash
                     )
                 if error_torrent_hash:
                     error_message = "Torrent hash(es): %s" % error_torrent_hash
             raise NotFound404Error(error_message)
 
-        elif response.status_code == 409:
+        if response.status_code == 409:
             # APIErrorType::Conflict
             raise Conflict409Error(response.text)
 
-        elif response.status_code == 415:
+        if response.status_code == 415:
             # APIErrorType::BadData
             raise UnsupportedMediaType415Error(response.text)
 
-        elif response.status_code >= 500:
+        if response.status_code >= 500:
             raise InternalServerError500Error(response.text)
 
-        elif response.status_code >= 400:
+        if response.status_code >= 400:
             # Unaccounted for errors from API
             raise HTTPError(response.text)
 
-    def verbose_logging(self, http_method, response, url):
+    def _verbose_logging(self, http_method, url, http_args, response):
         """Log verbose information about request. Can be useful during development."""
         if self._VERBOSE_RESPONSE_LOGGING:
             resp_logger = logger.debug
             max_text_length_to_log = 254
             if response.status_code != 200:
-                # log as much as possible in a error condition
+                # log as much as possible in an error condition
                 max_text_length_to_log = 10000
 
-            resp_logger("Request URL: (%s) %s" % (http_method.upper(), response.url))
-            resp_logger("Request Headers: %s" % response.request.headers)
+            resp_logger("Request URL: (%s) %s", http_method.upper(), response.url)
+            resp_logger("Request HTTP Args: %s", http_args)
+            resp_logger("Request Headers: %s", response.request.headers)
             if (
                 str(response.request.body) not in ("None", "")
                 and "auth/login" not in url
@@ -604,15 +702,13 @@ class Request(HelpersMixIn):
                     else len(response.request.body)
                 )
                 resp_logger(
-                    "Request body: %s%s"
-                    % (
-                        response.request.body[:body_len],
-                        "...<truncated>" if body_len >= 200 else "",
-                    )
+                    "Request body: %s%s",
+                    response.request.body[:body_len],
+                    "...<truncated>" if body_len >= 200 else "",
                 )
 
             resp_logger(
-                "Response status: %s (%s)" % (response.status_code, response.reason)
+                "Response status: %s (%s)", response.status_code, response.reason
             )
             if response.text:
                 text_len = (
@@ -621,11 +717,9 @@ class Request(HelpersMixIn):
                     else len(response.text)
                 )
                 resp_logger(
-                    "Response text: %s%s"
-                    % (
-                        response.text[:text_len],
-                        "...<truncated>" if text_len >= 80 else "",
-                    )
+                    "Response text: %s%s",
+                    response.text[:text_len],
+                    "...<truncated>" if text_len >= 80 else "",
                 )
         if self._PRINT_STACK_FOR_EACH_REQUEST:
             from traceback import print_stack
