@@ -43,7 +43,7 @@ class TorrentDictionary(Dictionary):
     :Usage:
         >>> from qbittorrentapi import Client
         >>> client = Client(host='localhost:8080', username='admin', password='adminadmin')
-        >>> # this are all the same attributes that are available as named in the
+        >>> # these are all the same attributes that are available as named in the
         >>> #  endpoints or the more pythonic names in Client (with or without 'transfer_' prepended)
         >>> torrent = client.torrents.info()[0]
         >>> torrent_hash = torrent.info.hash
@@ -869,7 +869,7 @@ class TorrentCategories(ClientCache):
     :Usage:
         >>> from qbittorrentapi import Client
         >>> client = Client(host='localhost:8080', username='admin', password='adminadmin')
-        >>> # this are all the same attributes that are available as named in the
+        >>> # these are all the same attributes that are available as named in the
         >>> #  endpoints or the more pythonic names in Client (with or without 'torrents_' prepended)
         >>> categories = client.torrent_categories.categories
         >>> # create or edit categories
@@ -1110,23 +1110,35 @@ class TorrentsAPIMixIn(Request):
             "firstLastPiecePrio": (None, is_first_last_piece_priority),
         }
 
-        files = self._normalize_torrent_files(torrent_files)
+        files_to_send, files_to_close = self._normalize_torrent_files(torrent_files)
 
-        return self._post(
-            _name=APINames.Torrents, _method="add", data=data, files=files, **kwargs
-        )
+        try:
+            return self._post(
+                _name=APINames.Torrents,
+                _method="add",
+                data=data,
+                files=files_to_send,
+                **kwargs
+            )
+        finally:
+            if files_to_close:
+                for fh in files_to_close:
+                    try:
+                        fh.close()
+                    except Exception as exp:
+                        logger.warning("Failed to close file: %r", exp)
 
     @staticmethod
     def _normalize_torrent_files(user_files):
         """
         Normalize the torrent file(s) from the user.
         The file(s) can be the raw bytes, file handle, filepath for a torrent file,
-        or a Sequence (e.g. list|set|tuple) of any of these "files".
+        or an iterable (e.g. list|set|tuple) of any of these "files".
         Further, the file(s) can be in a dictionary with the "names" of the torrents as the keys.
         These "names" can be anything...but are mostly useful as identifiers for each file.
         """
         if not user_files:
-            return None
+            return None, None
 
         prefix = "torrent__"
         # if it's string-like and not a list|set|tuple, then make it a list
@@ -1144,6 +1156,7 @@ class TorrentsAPIMixIn(Request):
         )
 
         files = {}
+        files_to_close = []
         for name, torrent_file in norm_files.items():
             try:
                 fh = None
@@ -1157,6 +1170,7 @@ class TorrentsAPIMixIn(Request):
                         )
                         if path.exists(filepath):
                             fh = open(filepath, "rb")
+                            files_to_close.append(fh)
                             name = path.basename(filepath)
                     except Exception:
                         fh = None
@@ -1173,6 +1187,7 @@ class TorrentsAPIMixIn(Request):
                         path.realpath(path.expanduser(str(torrent_file)))
                     )
                     fh = open(filepath, "rb")
+                    files_to_close.append(fh)
                     name = path.basename(filepath)
 
                 # if using default name, let Requests try to figure out the filename to send
@@ -1183,12 +1198,12 @@ class TorrentsAPIMixIn(Request):
                     raise TorrentFileNotFoundError(
                         errno.ENOENT, os_strerror(errno.ENOENT), torrent_file
                     )
-                elif io_err.errno == errno.EACCES:
+                if io_err.errno == errno.EACCES:
                     raise TorrentFilePermissionError(
                         errno.ENOENT, os_strerror(errno.EACCES), torrent_file
                     )
                 raise TorrentFileError(io_err)
-        return files or None
+        return files, files_to_close
 
     ##########################################################################
     # INDIVIDUAL TORRENT ENDPOINTS
@@ -1429,8 +1444,6 @@ class TorrentsAPIMixIn(Request):
         :param new_path: new path of file to rename (added in Web API v2.7)
         :return: None
         """
-        torrent_hash = torrent_hash
-
         # convert pre-v2.7 params to post-v2.7...or post-v2.7 to pre-v2.7
         # HACK: v4.3.2 and v4.3.3 both use web api v2.7 but old/new_path were introduced in v4.3.3
         if (
@@ -1496,7 +1509,7 @@ class TorrentsAPIMixIn(Request):
         :return: None
         """
         # HACK: v4.3.2 and v4.3.3 both use web api v2.7 but rename_folder was introduced in v4.3.3
-        if self._is_version_less_than("v4.3.3", self.app.version, lteq=True):
+        if self._is_version_less_than("v4.3.3", self.app_version(), lteq=True):
             data = {
                 "hash": torrent_hash,
                 "oldPath": old_path,
